package com.bea.medrec.service;

import com.bea.medrec.dao.PatientDao;
import com.bea.medrec.dao.UserDao;
import com.bea.medrec.domain.MedicalRecord;
import com.bea.medrec.domain.Patient;
import com.bea.medrec.domain.Registration;
import com.bea.medrec.domain.User;
import com.bea.medrec.exceptions.MedRecException;
import com.bea.medrec.utils.MedRecConstants;
import com.bea.medrec.utils.MedRecLog4jFactory;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.management.*;
import javax.naming.NameNotFoundException;
import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import weblogic.diagnostics.accessor.ColumnInfo;
import weblogic.management.ManagementException;
import weblogic.management.jmx.MBeanServerInvocationHandler;
import weblogic.management.mbeanservers.runtime.RuntimeServiceMBean;
import weblogic.management.runtime.ServerRuntimeMBean;
import weblogic.management.runtime.WLDFAccessRuntimeMBean;
import weblogic.management.runtime.WLDFDataAccessRuntimeMBean;
import weblogic.management.utils.AlreadyExistsException;

/**
 * <p>Service for all functionality related to adminstration management.</p>
 *
 * @author Copyright (c) @COPYRIGHT_CURRENTYEAR by BEA Systems. All Rights Reserved.
 */
public class AdminServiceImpl
    extends BaseService implements AdminService {

  private static Logger logger =
    MedRecLog4jFactory.getLogger(AdminServiceImpl.class.getName());

  // service beans
  protected PatientService patientService;
  protected RecordService recordService;
  protected MailService mailService;
  protected MessagingService messagingService;

  // wls mbean server connections
  protected MBeanServerConnection runtimeMbeanServerConnection;
  protected MBeanServerConnection domainMbeanServerConnection;

  // data access beans
  protected PatientDao patientDao;
  protected UserDao userDao;

  // injection setters
  public void setPatientDao(PatientDao pPatientDao) {
    this.patientDao = pPatientDao;
  }

  public void setUserDao(UserDao pUserDao) {
    this.userDao = pUserDao;
  }

  public void setMailService(MailService pMailService) {
    this.mailService = pMailService;
  }

  public void setMessagingService(MessagingService pMessagingService) {
    this.messagingService = pMessagingService;
  }

  public void setPatientService(PatientService patientService) {
    this.patientService = patientService;
  }

  public void setRecordService(RecordService recordService) {
    this.recordService = recordService;
  }

  public void setRuntimeMbeanServerConnection(
      MBeanServerConnection pRuntimeMbeanServerConnection) {
    this.runtimeMbeanServerConnection = pRuntimeMbeanServerConnection;
  }

  public void setDomainMbeanServerConnection(
      MBeanServerConnection pDomainMbeanServerConnection) {
    this.domainMbeanServerConnection = pDomainMbeanServerConnection;
  }

    //  A C T I V A T E   A C C O U N T   S T A T U S
  /**
  * <p>Activates a patients account for general access. A email notification
  * is sent to the patients.</p>
  *
  * @param pUser    User domain object.
  * @exception MedRecException
  */
  public void activateAccount(User pUser) throws MedRecException {
    // Load user object
    pUser = userDao.getByUsername(pUser.getUsername());
    logger.debug("Loaded user: "+pUser.toString());

    // Update account status. Set status to active.
    logger.debug("Updating "+pUser.getUsername()+" with account status "+
        MedRecConstants.USER_ACTIVE);
    pUser.setStatus(MedRecConstants.USER_ACTIVE);

    // Send mail object to be processed.
    mailService.sendApproveMail(pUser);
  }

  public void saveAdminUser(User pUser) throws MedRecException {
    try {
      saveAdminUserWork(pUser);
    } catch (MedRecException me) {
      throw me;
    } catch (Exception e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to save admin user: "+e.getMessage(), e);
    }
  }


  protected void saveAdminUserWork(User pUser)
      throws MedRecException, MalformedObjectNameException, ReflectionException,
        IOException, InstanceNotFoundException, MBeanException,
        AttributeNotFoundException {
    logger.debug("Adding admin user: "+pUser.toString());
    ObjectName service = new ObjectName("com.bea:Name=DomainRuntimeService,"+
        "Type=weblogic.management.mbeanservers.domainruntime.DomainRuntimeServiceMBean");
    ObjectName domainMBean
            = (ObjectName) domainMbeanServerConnection.getAttribute(service,
                "DomainConfiguration");
    ObjectName securityConfiguration
            = (ObjectName) domainMbeanServerConnection.getAttribute(domainMBean,
                "SecurityConfiguration");
    ObjectName defaultRealm
            = (ObjectName) domainMbeanServerConnection.getAttribute(
                securityConfiguration, "DefaultRealm");
    ObjectName[] atnProviders
            = (ObjectName[]) domainMbeanServerConnection.getAttribute(defaultRealm,
                "AuthenticationProviders");

    ObjectName defaultAtnProvider = null;

    for (ObjectName atnProvider : atnProviders) {
      String name = (String) domainMbeanServerConnection.getAttribute(
          atnProvider, "Name");
      if (name.equals("DefaultAuthenticator")) {
        logger.debug("Found the Default authenticator");
        defaultAtnProvider = atnProvider;
      }
    }

    try {
      domainMbeanServerConnection.invoke(
              defaultAtnProvider,
              "createUser",
              new Object[] {pUser.getUsername(), pUser.getPassword(),
                            "MedRec Admininistator"},
              new String[] {"java.lang.String", "java.lang.String",
                            "java.lang.String"}
      );
      logger.debug("Successfully invoked createUser");
    } catch (MBeanException ex) {
      Exception e = ex.getTargetException();
      if (e instanceof AlreadyExistsException) {
        logger.error("User, " + pUser.getUsername() + ", already exists.");
        throw new MedRecException("User, " + pUser.getUsername() +
            ", already exists.", e);
      } else {
        logger.debug(e);
         throw new MedRecException("Unable to create admin user: "+
             pUser.getUsername()+", password: "+pUser.getPassword(), e);
      }
    }

    try {
      domainMbeanServerConnection.invoke(
              defaultAtnProvider,
              "addMemberToGroup",
              new Object[] {"Administrators", pUser.getUsername()},
              new String [] {"java.lang.String", "java.lang.String"}
      );
      logger.debug("Successfully invoked addMemberToGroup");

      domainMbeanServerConnection.invoke(
              defaultAtnProvider,
              "addMemberToGroup",
              new Object[] {"MedRecAdmins", pUser.getUsername()},
              new String [] {"java.lang.String", "java.lang.String"}
      );
      logger.debug("Successfully invoked addMemberToGroup");
    } catch (MBeanException ex) {
      Exception e = ex.getTargetException();
      if (e instanceof NameNotFoundException) {
        logger.info("Invalid Group Name.");
      }
      logger.debug(e);
      throw new MedRecException("Unable to create admin user: "+e.getMessage(),
          e);
    }
  }

  public Object[] getLogRows(String pLogType, String pQueryStr)
      throws MedRecException {
    Object[] objs = null;
    try {
       objs = getLogRowsWork(pLogType, pQueryStr);
    } catch (Exception e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to get log rows for "+pLogType+
          " log type and '"+pQueryStr+"' query: "+e.getMessage(), e);
    }
    return objs;
  }

  protected Object[] getLogRowsWork(String pLogType, String pQueryStr)
     throws MalformedObjectNameException, ReflectionException,
        IOException, InstanceNotFoundException, MBeanException,
        AttributeNotFoundException {
    logger.debug("Getting log rows: log="+pLogType+", query="+pQueryStr);

    // local variables
    String cursor = null;
    Object[] rows = null;
    ObjectName wldfDataAccessRuntime =
        getWldfDataAccessRuntimeForLogType(pLogType);

    logger.debug("Query str: "+
        (isEmpty(pQueryStr) ? "<nothing specified>" : pQueryStr));
    cursor = (String) runtimeMbeanServerConnection.invoke(wldfDataAccessRuntime,
        "openCursor",
        new Object[] { pQueryStr },
        new String[] { "java.lang.String" });

    rows = (Object[]) runtimeMbeanServerConnection.invoke(
        wldfDataAccessRuntime,
        "fetch",
        new Object[] { cursor },
        new String[] { "java.lang.String" });

    logger.debug("Found "+rows.length+" log entries:");

    for (int i=0; i<rows.length; i++) {
      StringBuffer strBuff = new StringBuffer();
      strBuff.append("Row "+i+1+": ");
      Object[] cols = (Object[]) rows[i];
      for (int j=0; j<cols.length; j++) {
        strBuff.append(" Index "+j+"="+cols[j]);
      }
      logger.debug(strBuff.toString());
    }

    runtimeMbeanServerConnection.invoke(
        wldfDataAccessRuntime,
        "closeCursor",
        new Object[] { cursor },
        new String[] { "java.lang.String" });

    return rows;
  }

  protected ObjectName getWldfDataAccessRuntimeForLogType(String logType)
      throws MalformedObjectNameException, ReflectionException,
        IOException, InstanceNotFoundException, MBeanException,
        AttributeNotFoundException {
    ObjectName service = new ObjectName("com.bea:Name=RuntimeService,"+
        "Type=weblogic.management.mbeanservers.runtime.RuntimeServiceMBean");
    ObjectName serverRuntime = (ObjectName) runtimeMbeanServerConnection.getAttribute(
        service, "ServerRuntime");
    ObjectName wldfRuntime = (ObjectName) runtimeMbeanServerConnection.getAttribute(
        serverRuntime, "WLDFRuntime");
    ObjectName wldfAccessRuntime = (ObjectName) runtimeMbeanServerConnection.getAttribute(
        wldfRuntime, "WLDFAccessRuntime");

    return (ObjectName) runtimeMbeanServerConnection.invoke(
        wldfAccessRuntime, "lookupWLDFDataAccessRuntime",
        new Object[] { logType },
        new String[] { "java.lang.String" });
  }

  public WLDFAccessRuntimeMBean getDiagntosicMBean() throws MedRecException {
    logger.info("Getting WLDF runtime MBean.");
    RuntimeServiceMBean rtService = null;
    WLDFAccessRuntimeMBean dar = null;
    ObjectName rtSvcObjName = null;

    try {
      rtSvcObjName = new ObjectName(RuntimeServiceMBean.OBJECT_NAME);
      rtService = (RuntimeServiceMBean)
          MBeanServerInvocationHandler.newProxyInstance(
              runtimeMbeanServerConnection, rtSvcObjName);
      logger.debug("Obtained RuntimeServiceMBean.");
      ServerRuntimeMBean srt = rtService.getServerRuntime();
      logger.debug("Obtained ServerRuntimeMBean.");
      dar = srt.getWLDFRuntime().getWLDFAccessRuntime();
      logger.debug("Obtained WLDFAccessRuntimeMBean.");
    } catch (MalformedObjectNameException e) {
      logger.error("Unable to get diagnostic mbean: "+
            e.getMessage(), e);
      throw new MedRecException("Unable to get diagnostic mbean: "+
            e.getMessage(), e);
    } catch (Exception e) {
      logger.error("Unable to get diagnostic mbean: "+
            e.getMessage(), e);
      throw new MedRecException("Unable to get diagnostic mbean: "+
            e.getMessage(), e);
    }
    return dar;
  }

  public String[] getLogTypes() throws MedRecException {
    String[] logTypes = null;
    WLDFAccessRuntimeMBean dar = null;
    try {
      dar = getDiagntosicMBean();
      if (dar == null)
        throw new MedRecException("Unable to obtain WLDFAccessRuntimeMBean, "+
            "getDiagntosicMBean returned null");
      logTypes = dar.getAvailableDiagnosticDataAccessorNames();
    } catch (ManagementException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to get log types: "+
            e.getMessage(), e);
    }
    return logTypes;
  }

  public ColumnInfo[] getLogColumnInfo(String pLogType)
      throws MedRecException {
    WLDFAccessRuntimeMBean daigRuntime = getDiagntosicMBean();
    WLDFDataAccessRuntimeMBean accessor = null;
    ColumnInfo[] colmInfo = null;
    try {
      accessor = daigRuntime.lookupWLDFDataAccessRuntime(pLogType);
      colmInfo = accessor.getColumns();
    } catch (ManagementException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to get log column info: "+
            e.getMessage(), e);
    }
    return colmInfo;
  }

    //  A D D   P A T I E N T
  /**
  * <p>Presists a new patient if patient does not already exist.</p>
  *
  * @param pPatient    Patient value objects.
  * @exception MedRecException
  */
  public void savePatientUser(Patient pPatient, User pUser)
      throws MedRecException {
    logger.debug("Add Patient User: "+pPatient.toString());

    // Check to see if user already exists.  Is so, return id.
    try {
      if (userDao.getByUsername(pUser.getUsername()) == null) {
        logger.info("User with username="+pUser.getUsername()+"exists.");
        throw new MedRecException("User with username="+pUser.getUsername()+
            "exists.");
      } else {
        logger.debug("User not found. Creating new patient user.");
        userDao.save(pUser);
        patientDao.save(pPatient);
        logger.debug("Patient User created.");
      }
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to save user/patient: "+
            e.getMessage(), e);
    }
  }

  //  C R E A T E   P A T I E N T   &   L O G I N
  /**
  * <p>Create new patient account, login and groups,
  * and store revelent patient data.</p>
  *
  * @param pPatient    Patient value objects.
  */
  public Integer createPatientAccount(User pUser, Patient pPatient)
      throws MedRecException {
    logger.info("Creating new patient account- login and patient data.");
    try {
      userDao.save(pUser);
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to create account: "+
            e.getMessage(), e);
    }
    Patient newPatient = patientService.savePatient(pPatient);
    return newPatient.getId();
  }

  //  C R E A T E   P A T I E N T   &   L O G I N
  /**
  * <p>Create new patient account, login and groups,
  * and store revelent patient data.</p>
  *
  * @param pPatient    Patient value objects.
  */
  public Patient createPatientAccount(Patient pPatient)
      throws MedRecException {
    
    logger.info("Creating new patient account- login and patient data.");
    try {
      return patientService.savePatient(pPatient);
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to create account: "+
            e.getMessage(), e);
    }
  }

    //  D E N Y   A C C O U N T   S T A T U S
  /**
  * <p>Deny new user registration.  Sets account status to "DENY".</p>
  *
  * @param pUser    Username of account to be denied.
  * @exception MedRecException
  */
  public void denyAccountStatus(User pUser)
      throws MedRecException {
    pUser.setStatus("DENY");
    try {
      userDao.save(pUser);
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to set account status to DENY: "+
            e.getMessage(), e);
    }
    mailService.sendDenyMail(pUser);
  }

    //  F I N D   A D M I N   B Y   S T A T U S
  /**
* <p>Finds patients by status "NEW".</p>
*
* @return Collection  Collection of patient value objects.
* @exception MedRecException
*/
  public List getNewPatients() throws MedRecException {
    List patients = null;
    try {
      patients = patientDao.getByStatus("NEW");
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to get new patients: "+
            e.getMessage(), e);
    }
    return patients;
  }

      //  F I N D   U S E R
  /**
  * <p>FIXME</p>
  *
  * @param pUser    User domain objects.
  * @return User
  */
  public User getUser(User pUser) throws MedRecException {
    User user = null;
    if (isNotEmpty(pUser.getUsername())) {
      logger.debug("Getting user "+pUser.getUsername());
      try {
        user = userDao.getByUsername(pUser.getUsername());
        logger.debug("Got "+user.toString());
      } catch (DataAccessException e) {
        throw new MedRecException("Unable to get user: "+
            e.getMessage(), e);
      }
    }

    return user;
  }

  //   I N S E R T   M E D I C A L   R E C O R D
  /**
  * <p>Creates new patient account and stores associate records.
  * Used by the XML upload feature where medical institutions
  * pass all held record information for a particular patient.</p>
  *
  * @param pMedicalRecords   List of MedicalRecord objects
  * @exception MedRecException
  */
  public void saveMedicalRecord(List<MedicalRecord> pMedicalRecords)
      throws MedRecException {
    logger.info("Insert medical record.");

    if (pMedicalRecords == null)
      throw new MedRecException("Medical records not found.");

    // Declare local variables.
    Patient patient = null;
    Iterator itr = pMedicalRecords.iterator();
    while (itr.hasNext()) {
      // Iterator thru medical collection.  Get physician id.
      MedicalRecord medicalRecord = (MedicalRecord) itr.next();

      // Get patient object.  Set status.
      // Then create patient and patient login data.
      createPatientAccount(medicalRecord.getPatient());

      patient = patientService.getPatient(medicalRecord.getPatient());
      
      // inserted medical records
      recordService.saveRecords(medicalRecord.getRecords(), patient);
    }
  }


    //    P R O C E S S   R E G I S T R A T I O N
  /**
  * <p>Start registration process by placing registration object on queue.
  * The object is eventually picked up and acted upon.</p>
  *
  * @param pReg    Registration value objects.
  * @exception MedRecException
  */
  public void saveRegistration(Registration pReg) throws MedRecException {
    logger.debug("Sending registration to queue");
    messagingService.sendToQueue(pReg);
  }

    //    P R O C E S S   R E G I S T R A T I O N
  /**
  * <p>Start XML file upload process by placing filename on queue.
  * The object is eventually picked up and acted upon.</p>
  *
  * @param pUploadFile    XML file to be uploaded.
  * @exception MedRecException
  */
  public void processPatientRecordUpload(File pUploadFile)
      throws MedRecException {
    logger.debug("Sending xml filename to queue");
    messagingService.sendToQueue(pUploadFile);
  }

    //  U P D A T E   A C C O U N T   S T A T U S
  /**
  * <p>Update user account status to given status.</p>
  *
  * @param pUser    User domain objects.
  * @exception MedRecException
  */
  public void updateAccountStatus(User pUser)
      throws MedRecException {
    logger.debug("Updating "+pUser.getUsername()+" with account status "+
        pUser.getStatus());
    try {
      userDao.save(pUser);
    } catch (DataAccessException e) {
      logger.error("Exception occur: "+e.getMessage(), e);
      throw new MedRecException("Unable to undate account status: "+
            e.getMessage(), e);
    }
  }

}