package com.bea.medrec.service;

import com.bea.medrec.dao.PatientDao;
import com.bea.medrec.domain.Patient;
import com.bea.medrec.domain.Registration;
import com.bea.medrec.domain.User;
import com.bea.medrec.exceptions.MedRecException;
import com.bea.medrec.utils.MedRecConstants;
import com.bea.medrec.utils.MedRecLog4jFactory;
import java.util.Collections;
import java.util.List;
import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;

/**
 * <p>
 * Service for all functionality related to patient management.
 * </p>
 * 
 * @author Copyright (c)
 * @COPYRIGHT_CURRENTYEAR by BEA Systems. All Rights Reserved.
 */
public class PatientServiceImpl extends BaseService implements PatientService {
    private static Logger logger = MedRecLog4jFactory.getLogger(PatientServiceImpl.class.getName());

    // dao and service beans
    protected AdminService adminService;

    protected PatientDao patientDao;

    // injection setter
    public void setAdminService(AdminService adminService) {
        this.adminService = adminService;
    }

    public void setPatientDao(PatientDao patientDao) {
        this.patientDao = patientDao;
    }

    // P U B L I C M E T H O D S
    /**
     * 
     */
    public Patient authenticate(User user) throws MedRecException {
        Patient patient = null;
        if (user != null && isNotEmpty(user.getUsername())) {
            try {
                patient = patientDao.getByUsername(user.getUsername());
            } catch (DataAccessException e) {
                logger.error("Exception occurred: " + e.getMessage(), e);
                throw new MedRecException("Unable to get user by username: " + e.getMessage());
            }
        } else {
            throw new MedRecException("Username not specified.");
        }
        return patient;
    }

    /**
     * 
     */
    public Patient getPatient(Patient pPatient) throws MedRecException {
        Patient patient = null;
        try {
            if (pPatient.getUser()!=null&&isNotEmpty(pPatient.getUser().getUsername())) {
                List patients = patientDao.getByEmail(pPatient.getUser().getUsername());
                patient = (Patient) patients.get(0);
            } else if (isNotEmpty(pPatient.getSsn())) {
                patient = patientDao.getBySsn(pPatient.getSsn());
            } else if (pPatient != null && isNotEmpty(pPatient.getId())) {
                patient = patientDao.getById(pPatient.getId());
            } else {
                throw new MedRecException("Patient search criteria not specified.");
            }
        } catch (DataAccessException e) {
            logger.error("Exception occurred: " + e.getMessage(), e);
            throw new MedRecException("Unable to get patient: " + e.getMessage());
        }
        return patient;
    }

    /**
     * 
     */
    public List getPatients(Patient pPatient) throws MedRecException {
        if (pPatient == null)
            return Collections.EMPTY_LIST;

        List patients = null;
        try {
            if (isNotEmpty(pPatient.getUser().getUsername())) {
                patients = patientDao.getByEmail(pPatient.getUser().getUsername());
            } else if (isNotEmpty(pPatient.getLastName()) && isNotEmpty(pPatient.getFirstName())) {
                patients = patientDao.getByLastNameFirstName(pPatient.getLastName(), pPatient.getFirstName());
            } else if (isNotEmpty(pPatient.getLastName())) {
                patients = patientDao.getByLastName(pPatient.getLastName());
            } else {
                throw new MedRecException("Search criteria not specified.");
            }
        } catch (DataAccessException e) {
            logger.error("Exception occurred: " + e.getMessage(), e);
            throw new MedRecException("Unable to get patients: " + e.getMessage(), e);
        }

        return patients;
    }

    public List getPatientByWildLastName(Patient patient) throws MedRecException {
        if (patient == null)
            return Collections.EMPTY_LIST;

        List patients = null;
        if (patient != null && isNotEmpty(patient.getLastName())) {
            try {
                patients = patientDao.getByLastNameWild(patient.getLastName());
            } catch (DataAccessException e) {
                logger.error("Exception occurred: " + e.getMessage(), e);
                throw new MedRecException("Unable to get patient: " + e.getMessage(), e);
            }
        } else {
            throw new MedRecException("Search criteria not specified.");
        }

        return patients;
    }

    // R E G I S T R A T I O N
    
    /**
     * <p>
     * Registers new patient account assign status to NEW. Typically this
     * functionality is used by online registrants that must go through a MedRec
     * adminstration approval process.
     * </p>
     * 
     * @param pReg
     *            Patient value objects.
     */
    public void processNewRegistration(Registration registration) {
        try {
            // Assign account status to NEW.
            User user = registration.getUser();
            Patient patient = registration.getPatient();
            patient.setUser(user);
            // user.setPatient(patient);
            user.setStatus(MedRecConstants.USER_NEW);

            processRegistration(registration);
        } catch (Exception ex) {
            logger.error("Unable to register the following user: \n" + registration.toString(), ex);
            logger.error("Sending registration failure to MedRec admin");
            // Send registration to error queue for manual
            // processing by MedRec admintration.
        }
    }

    /**
     * <p>
     * Main patient registration process method. This method creates a login
     * account, assigns patient designated groups, and stores pertinent patient
     * data such as name, dob, ssn, etc. This method acts as the worker to
     * different types of registration.
     * </p>
     * 
     * @param registration
     *            Registration value objects.
     * @throws MedRecException
     */
    public void processRegistration(Registration registration) throws MedRecException {
        if (registration == null)
            throw new MedRecException("Registration is null");
        if (registration.getPatient() == null || registration.getUser() == null)
            throw new MedRecException("Patient or user is null");

        logger.debug("Registering Patient: " + registration);

        // declare local variables.
        Patient patient = registration.getPatient();
        // Create user, groups and patient information.
        patient = adminService.createPatientAccount(patient);
        logger.debug("Registration successful.");
    }

    // U P D A T E P A T I E N T
    /**
     * <p/>Saves new or updated patient.
     * </p>
     * 
     * @param pPatient
     *            Patient value objects.
     * @return Patient
     * @throws MedRecException
     * @throws MedRecException
     */
    public Patient savePatient(Patient pPatient) throws MedRecException {
        logger.debug("Saving Patient: " + pPatient.toString());
        try {
            return patientDao.save(pPatient);
        } catch (DataAccessException e) {
            logger.error("Exception occurred: " + e.getMessage(), e);
            throw new MedRecException("Unable to save patient: " + e.getMessage(), e);
        }
    }

    public Patient updatePatient(Patient pPatient) throws MedRecException {
        logger.debug("Saving Patient: " + pPatient.toString());
        try {
            return patientDao.update(pPatient);
        } catch (DataAccessException e) {
            logger.error("Exception occurred: " + e.getMessage(), e);
            throw new MedRecException("Unable to update patient: " + e.getMessage(), e);
        }

    }
}