package com.bea.medrec.service;

import com.bea.medrec.dao.PrescriptionDao;
import com.bea.medrec.dao.RecordDao;
import com.bea.medrec.domain.Patient;
import com.bea.medrec.domain.Prescription;
import com.bea.medrec.domain.Record;
import com.bea.medrec.domain.RecordsSummary;
import com.bea.medrec.exceptions.MedRecException;
import com.bea.medrec.utils.MedRecLog4jFactory;
import java.util.Collections;
import java.util.List;
import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;

/**
 * <p>Service for all functionality related to record management.</p>
 *
 * @author Copyright (c) @COPYRIGHT_CURRENTYEAR by BEA Systems. All Rights Reserved.
 */
public class RecordServiceImpl
    extends BaseService implements RecordService {
  private static Logger logger =
    MedRecLog4jFactory.getLogger(RecordServiceImpl.class.getName());

  RecordDao recordDao;
  PrescriptionDao prescriptionDao;

  public void setRecordDao(RecordDao pRecordDao) {
    this.recordDao = pRecordDao;
  }

  public void setPrescriptionDao(PrescriptionDao pPrescriptionDao) {
    this.prescriptionDao = pPrescriptionDao;
  }

  //  A D D   R E C O R D
  /**
 * <p>Adds a record, including vital signs and prescriptions, by
 * calling the appropriate entity beans.</p>
 *
 * @param pRecord Complete record value object to be added
 */
  public Record saveRecord(Record pRecord) throws MedRecException {
    logger.debug("Adding record: "+pRecord.toString());
    Integer recordId = recordDao.save(pRecord);
    pRecord.setId(recordId);
    return pRecord;
  }

  public void saveRecords(List pRecord) throws MedRecException {
    logger.debug("Adding "+pRecord.size()+" records.");
    for (Object aPRecord : pRecord) {
      saveRecord((Record) aPRecord);
    }
  }

  public void saveRecords(List pRecord, Patient pPatient)
      throws MedRecException {
    logger.debug("Adding "+pRecord.size()+" records.");
    for (Object aPRecord : pRecord) {
      Record record = (Record) aPRecord;
      record.setPatientId(pPatient.getId());
      pPatient.addRecord(record);
      
      for (Object object: record.getPrescriptions()){
        Prescription prescription=(Prescription)object;
        prescription.setPatientId(pPatient.getId());
        pPatient.addPrescription (prescription);
      }
      
      saveRecord(record);
    }
  }

  //  G E T   P R E S C R I P T I O N S
  /**
  * <p>Retrieves all prescriptions for a patient.</p>
  *
  * @param pPatient Id of the prescription.
  * @return Collection  Collection of prescription value objects.
  * @throws MedRecException
  */
  public List getPrescriptions(Patient pPatient) throws MedRecException {
    List prescriptions = null;

    if (pPatient != null && isNotEmpty(pPatient.getId())) {
      logger.debug("Getting prescriptions for patient: "+pPatient.toStringLite());
      try {
        prescriptions = prescriptionDao.getByPatientId(pPatient.getId());
      } catch (DataAccessException e) {
        logger.error("Exception occurred: "+e.getMessage(), e);
        throw new MedRecException("Unable to get prescriptions for patient id "+
            pPatient.getId()+": "+e.getMessage(), e);
      }
    } else {
      prescriptions = Collections.EMPTY_LIST;
    }

    return prescriptions;
  }

  //  G E T   A L L   R E C O R D S   F O R   A   P A T I E N T
  /**
  * <p>Retrieves a records for a patient.  Records return are lite weight.</p>
  *
  * @param pPatient patient object.
  * @return Collection  Collection of lite record value objects.
  * @throws MedRecException
  */
  public List getRecords(Patient pPatient) throws MedRecException {
    List records = null;
    if (pPatient != null && isNotEmpty(pPatient.getId())) {
      logger.debug("Getting records for patient: "+pPatient.toStringLite());
      try {
        records = (List)recordDao.getByPatientId(pPatient.getId());
      } catch (DataAccessException e) {
        logger.error("Exception occurred: "+e.getMessage(), e);
        throw new MedRecException("Unable to get records for patient id "+
            pPatient.getId()+": "+e.getMessage(), e);
      }
    } else {
      records = Collections.EMPTY_LIST;
    }
    return records;
  }

  //  G E T   A   R E C O R D
  /**
  * <p>Retrieves all information for a visit given a record id.
  * This includes vital signs, the record, and any prescriptions.</p>
  *
  * @param pRecordId Id of a record.
  * @return Record  Record value object.
  * @throws MedRecException
  */
  public Record getRecord(Integer pRecordId) throws MedRecException {
    logger.debug("Getting record for record id: "+pRecordId);
    Record record = null;
    try {
      record = recordDao.getById(pRecordId);
    } catch (DataAccessException e) {
      logger.error("Exception occurred: "+e.getMessage(), e);
      throw new MedRecException("Unable to get record "+pRecordId+": "+
          e.getMessage(), e);
    }
    return record;
  }

  //  G E T   R E C O R D S   S U M M A R Y
  /**
  * <p>Retrieves all medical record summary.
  * This includes a collection of abbreviated records, and
  * a collection of current and recent prescriptions.</p>
  *
  * @param pPatient Id of the patient.
  * @return RecordsSummary
  * @throws MedRecException
  */
  public RecordsSummary getRecordsSummary(Patient pPatient)
      throws MedRecException {
    logger.debug("Getting records and prescriptions for patient id: "+
        pPatient.toString());
    if (isNotEmpty(pPatient.getId())) {
      List recordList = getRecords(pPatient);
      List prescriptionList = getPrescriptions(pPatient);
      Record[] records = (Record[])recordList.toArray(
          new Record[recordList.size()]);
      Prescription[] prescriptions =
          (Prescription[])prescriptionList.toArray(
              new Prescription[prescriptionList.size()]);
      return new RecordsSummary(records, prescriptions);
    } else {
      return new RecordsSummary();
    }
  }
}