package com.bea.medrec.service;

import com.bea.medrec.domain.*;
import com.bea.medrec.utils.MedRecLog4jFactory;
import com.bea.medrec.xml.types.*;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.sql.Date;
import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlException;

/**
 * FIXME
 */
public class RecordXmlConvertorImpl
    implements RecordXmlConvertor {

  private static Logger logger =
      MedRecLog4jFactory.getLogger(RecordXmlConvertorImpl.class.getName());

  private MedicalRecordsDocument medrecDoc;

  /**
   * <p>Parses and validates the document that will be converted.</p>
   *
   * @param stream InputStream
   * @throws IOException
   */
  public void parse(InputStream stream) throws IOException, XmlException {
    if (stream == null) {
      throw new IOException("InputStream for XmlBean is null.");
    } else {
      medrecDoc = MedicalRecordsDocument.Factory.parse(stream);
    }

    if (!medrecDoc.validate()) {
      logger.info("Document not valid");
    } else {
      logger.info("Document is valid");
    }
  }

  /**
   * <p>Primary method of this class used to convert one type of bean
   * to another.</p>
   *
   * @return Collection
   */
  public Collection<MedicalRecord> getMedicalRecords() {
    MedicalRecordType medrec = medrecDoc.getMedicalRecords();
    Collection<MedicalRecord> medrecCol = new ArrayList<MedicalRecord>();

    String srcId = medrec.getSrcId();
    String srcName = medrec.getSrcName();

    MedicalVisitType[] visits = medrec.getMedicalVisitArray();

    for (MedicalVisitType visit : visits) {
      MedicalRecord medicalRecordVO = convertPatientVisit(visit,
          srcId, srcName);
      medrecCol.add(medicalRecordVO);
    }

    logger.debug(medrecCol.toString());
    return medrecCol;
  }

  /**
   * <p>Converts a MedicalVisit to a MedicalRecord with all the trimmings.</p>
   *
   * @param visit   MedicalVisitType
   * @param srcId   String
   * @param srcName String
   * @return MedicalRecord
   */
  private MedicalRecord convertPatientVisit(MedicalVisitType visit,
                                            String srcId, String srcName) {
    logger.debug("convertPatientVisit called...");
    MedicalRecord medicalRecordVO = new MedicalRecord(Integer.valueOf(srcId),
        srcName);
    Patient patient = null;

    patient = convertPatient(visit.getPatient());
    
    medicalRecordVO.setPatient(patient);
    medicalRecordVO.setUser(patient.getUser());

    RecordType[] records = visit.getRecordArray();

    for (RecordType record : records) {
      Record recordVO = convertRecord(record);
      medicalRecordVO.addRecord(recordVO);
    }

    return medicalRecordVO;
  }

  /**
   * <p>Converts the Patient bean.</p>
   *
   * @param patientType PatientInfoType
   * @return Patient
   */
  private Patient convertPatient(PatientInfoType patientType) {

    logger.debug("convertPatient called...");
    Patient patient = new Patient();

    patient.setSsn(patientType.getSsn());
    patient.setFirstName(patientType.getPatientName().getFirstName());
    patient.setMiddleName(patientType.getPatientName()
        .getMiddleName());
    patient.setLastName(patientType.getPatientName().getLastName());
    patient.setDob(patientType.getDob().getTime());
    patient.setPhone(patientType.getPhone());
    logger.debug("Gender ="+patientType.getGender()+".");
    String gender = (patientType.getGender().toString().equals("male") ? "Male"
        : "Female");
    patient.setGender(gender);
   
    User user = new User();
    user.setUsername(patientType.getEmail());
    user.setPassword(patientType.getSsn());
    user.setStatus("NEW");   
    patient.setUser(user);
    
    Address address = convertAddress(patientType.getAddress());
    patient.setAddress(address);
    
    return patient;
  }

  /**
   * <p>Converts the Record bean.</p>
   *
   * @param recordType RecordType
   * @return Record
   */
  private Record convertRecord(RecordType recordType) {
    logger.debug("convertRecord called...");
    Record record = new Record();

    record.setRecordDate(recordType.getDate().getTime());
    record.setSymptoms(recordType.getSymptoms());
    record.setDiagnosis(recordType.getDiagnosis());
    record.setNotes(recordType.getNotes());

    Physician physician = convertPhysician(recordType.getPhysician());
    record.setPhysician(physician);
    
    VitalSigns vitals = convertVitals(recordType.getVitalSigns());
    record.setVitalSigns(vitals);
    
    PrescriptionType[] prescriptionTypes = recordType.getPrescriptionArray();

    if (prescriptionTypes.length > 0) {
      Set prescriptions = new HashSet(prescriptionTypes.length);
      for (PrescriptionType prescriptionType : prescriptionTypes) {
        prescriptions.add(convertPrescription(prescriptionType, record));
      }
      record.setPrescriptions((Prescription[])prescriptions.toArray(new Prescription[prescriptions.size()]));
    }
    
    return record;
  }

  /**
   * <p>Converts the Physician bean.</p>
   *
   * @param physicianType VitalSignsType
   * @return VitalSigns
   */
  private Physician convertPhysician(PhysicianInfoType physicianType) {
    logger.debug("convertPhysician called...");
    Physician physician = new Physician();
    physician.setFirstName(physicianType.getFirstName());
    physician.setMiddleName(physicianType.getMiddleName());
    physician.setLastName(physicianType.getLastName());
    physician.setPhone(physicianType.getPhone());
    physician.setEmail(physicianType.getEmail());
    return physician;
  }

  /**
   * <p>Converts the VitalSigns bean.</p>
   *
   * @param vitalsType VitalSignsType
   * @return VitalSigns
   */
  private VitalSigns convertVitals(VitalSignsType vitalsType) {
    logger.debug("convertVitals called...");
    VitalSigns vitals = new VitalSigns();

    vitals.setWeight(new Integer(vitalsType.getWeight().getAmount()
        .toString()));
    vitals.setHeight(new Integer(vitalsType.getHeight().getAmount()
        .toString()));
    vitals.setTemperature(String.valueOf(vitalsType.getTemperature()
        .getAmount()));
    vitals.setBloodPressure(vitalsType.getBloodPressure()
        .getSystolic()+"/"
       +vitalsType.getBloodPressure().getDiastolic());
    vitals.setPulse(vitalsType.getPulse().toString());

    return vitals;
  }

  /**
   * <p>Converts the Prescription bean.</p>
   *
   * @param prescriptionType PrescriptionType
   * @return Prescription
   */
  private Prescription convertPrescription(PrescriptionType prescriptionType, Record record) {
    logger.debug("convertPrescription called...");
    Prescription prescription = new Prescription();

    prescription.setDatePrescribed(prescriptionType.getDate().getTime());
    prescription.setDrug(prescriptionType.getDrug());
    prescription.setDosage(prescriptionType.getDosage());
    prescription.setFrequency(prescriptionType.getFrequency());
    prescription.setRefillsRemaining(
        new Integer(prescriptionType.getRefills().toString()));
    prescription.setInstructions(prescriptionType.getInstructions());

    return prescription;
  }

  /**
   * <p>Converts the Address bean.</p>
   *
   * @param addressType AddressType
   * @return Address
   */
  private Address convertAddress(AddressType addressType) {
    logger.debug("convertAddress called...");
    Address addr = new Address();

    addr.setStreet1(addressType.getStreet1());
    addr.setStreet2(addressType.getStreet2());
    addr.setCity(addressType.getCity());
    addr.setState(addressType.getState());
    addr.setZip(addressType.getZip());
    addr.setCountry(addressType.getCountry());
    
    return addr;
  }

  /**
   * <p>For testing.</p>
   *
   * @param args String[]
   */
  public static void main(String[] args) {
    RecordXmlConvertor b2b = new RecordXmlConvertorImpl();
    try {
      b2b.parse(null);
      b2b.getMedicalRecords();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
