package com.bea.medrec.webservice;

import com.bea.medrec.domain.Patient;
import com.bea.medrec.domain.Record;
import com.bea.medrec.domain.RecordsSummary;
import com.bea.medrec.utils.MedRecLog4jFactory;
import com.bea.medrec.exceptions.MedRecException;
import java.util.ArrayList;
import java.util.List;
import javax.jws.WebMethod;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import org.apache.log4j.Logger;
import weblogic.jws.WLHttpTransport;

// Standard JWS annotation that specifies that the name of the Web Service is
// "MedRecWebServices", its public service name is "MedRecWebServices", and the
// targetNamespace used in the generated WSDL is "http://www.bea.com/medrec"
@WebService(name = "MedRecWebServicesPortType",
    serviceName = "MedRecWebServices",
    targetNamespace = "http://www.bea.com/medrec")

// Standard JWS annotation that specifies this is a document-literal-wrapped
// Web Service
@SOAPBinding(style=SOAPBinding.Style.DOCUMENT,
             use=SOAPBinding.Use.LITERAL,
             parameterStyle=SOAPBinding.ParameterStyle.WRAPPED)

// WebLogic-specific JWS annotation that specifies the port name is "MedRecWebServicesPort",
// and the context path and service URI used to build the URI of the Web
// Service is "ws_medrec/MedRecWebServices"
@WLHttpTransport(portName = "MedRecWebServicesPort",
    contextPath = "ws_medrec",
    serviceUri = "MedRecWebServices")

/**
 * <p>MedRecWebServices provides an interface for all MedRec Web services.</p>
 *
 * @author Copyright (c) @COPYRIGHT_CURRENTYEAR by BEA Systems. All Rights Reserved.
 */
public class MedRecWebServices
    extends MedRecBaseWebServices {
  private static Logger logger =
      MedRecLog4jFactory.getLogger(MedRecWebServices.class.getName());

  //  G E T   A   R E C O R D
  /**
   * <p>Retrieves all information for a visit given a record id.
   * This includes vital signs, the record, and any prescriptions.</p>
   *
   * @param pRecordId Id of the record.
   * @return Record Record value object
   * @throws Exception
   */
  @WebMethod()
  public Record getRecord(Integer pRecordId) throws Exception {
    logger.info("Getting record.");
    logger.debug("Record id: "+pRecordId);

    // Declare local variables.
    Record record = null;

    try {
      record = getMedRecServiceFacade().getRecord(pRecordId);
    } catch (MedRecException e) {
      logger.error("Exception occurred: "+e.getMessage(), e);
      throw e;
    }
    return record;
  }

  //  G E T   R E C O R D S   S U M M A R Y
  /**
   * <p>Retrieves all medical record summary.
   * This includes a List of abbreviated records, and
   * a List of current and recent prescriptions.</p>
   *
   * @param pPatientId Id of the record.
   * @return RecordsSummary Record Summary value object
   * @throws Exception
   */
  @WebMethod()
  public RecordsSummary getRecordsSummary(Integer pPatientId)
      throws Exception {
    logger.info("Getting records summary for patient id: "+pPatientId);

    // Declare local variables.
    RecordsSummary recSumary = null;

    try {
      recSumary = getMedRecServiceFacade().getRecordsSummary(
          new Patient(pPatientId));
    } catch (MedRecException e) {
      logger.error("Exception occurred: "+e.getMessage(), e);
      throw e;
    }
    return recSumary;
  }

  //  G E T   P A T I E N T
  /**
   * <p>Find patient by last name.  Wild card search- %lastName%.</p>
   *
   * @param pPatient Patient object
   * @return Patient[]  Array of Patient value objects.
   * @throws Exception
   */
  @WebMethod()
  public Patient[] searchPatients(Patient pPatient)
      throws Exception {
    logger.info("Search patients.");
    logger.debug("Patient: "+pPatient.toString());

    // Declare local variables.
    List patientList = new ArrayList();
    Patient[] patients = null;
    try {
      if (pPatient.getLastName() != null) {
        patientList = getMedRecServiceFacade().getPatientByWildLastName(pPatient);
      } else if (pPatient.getSsn() != null) {
        Patient newPatient = getMedRecServiceFacade().getPatient(pPatient);
        if (newPatient!=null) patientList.add(newPatient);
      }
      if (patientList != null && patientList.size() > 0) {
        patients = (Patient[])patientList.toArray(
            new Patient[patientList.size()]);
      } else {
        patients = new Patient[0];
      }
    } catch (MedRecException e) {
      logger.error("Exception occurred: "+e.getMessage(), e);
      throw e;
    }
    return patients;
  }
}