package com.bea.physician.service;

import com.bea.medrec.webservices.client.*;
import com.bea.physician.domain.*;
import com.bea.physician.exceptions.PhysicianException;
import com.bea.physician.utils.PhysicianLog4jFactory;
import com.bea.physician.webservices.client.*;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.apache.log4j.Logger;

/**
 * <p>Session Bean implementation for physician functionality including
 * access MedRec web services.</p>
 *
 * @author Copyright (c) @COPYRIGHT_CURRENTYEAR by BEA Systems. All Rights Reserved.
 */
public class PhysicianServiceImpl
    extends BaseService implements PhysicianService {

  private static Logger logger =
      PhysicianLog4jFactory.getLogger(PhysicianServiceImpl.class.getName());

  // instance variables
  protected MedRecWebServicesPortType syncMedRecWebServicesPortType;
  protected PhysicianWebServicesPortType reliableClientWebServicesPortType;
  protected String reliableServiceUrl;

  // injected setters
  public void setSyncMedRecWebServicesPortType(
      MedRecWebServicesPortType pSyncMedRecWebServicesPortType) {
    this.syncMedRecWebServicesPortType = pSyncMedRecWebServicesPortType;
  }

  public void setReliableClientWebServicesPortType(
      PhysicianWebServicesPortType pReliableClientWebServicesPortType) {
    this.reliableClientWebServicesPortType = pReliableClientWebServicesPortType;
  }

  public void setReliableServiceUrl(String pReliableServiceUrl) {
    this.reliableServiceUrl = pReliableServiceUrl;
  }

  // Public Methods

  //  A D D   R E C O R D
  /**
   * <p>Accesses MedRec Web service adding a record, including
   * vital signs and prescriptions.</p>
   *
   * @param pPhysician
   * @param pRecord
   * @throws PhysicianException
   */
  public void addRecord(Physician pPhysician, Record pRecord)
      throws PhysicianException {
    logger.info("Adding record to patient id: "+pRecord.getPatientId());
    logger.debug(pRecord.toString());
    try {
      pRecord.setPhysician(pPhysician);
      logger.info("Adding record reliably, asynchronously");
      logger.debug("Reliable WS url: "+reliableServiceUrl);
      reliableClientWebServicesPortType.addRecord(reliableServiceUrl,
          toMedRecRecord(pRecord));
    } catch (Exception e) {
      logger.error(e);
      throw new PhysicianException("Unable to add record: "+e.getMessage(), e);
    }
  }

  //  G E T   A   R E C O R D
  /**
   * <p>Accesses MedRec Web service retrieving all information
   * for a visit given a record id. This includes vital signs,
   * the record, and any prescriptions.</p>
   *
   * @param pRecordId record id
   * @return Record  record object
   * @throws PhysicianException
   */
  public Record getRecord(Integer pRecordId) throws PhysicianException {
    logger.info("Getting record.");
    logger.debug("Record Id: "+pRecordId);
    com.bea.medrec.domain.Record record = null;
    try {
      // Get record from MedRec.
      record = syncMedRecWebServicesPortType.getRecord(pRecordId.intValue());
      if (record == null) throw new PhysicianException("Return record is null");
      logger.debug("Got record from MedRec.");
    } catch (Exception e) {
      logger.error(e);
      throw new PhysicianException("Unable to get record: "+e.getMessage(), e);
    }
    return toPhysicianRecord(record);
  }

  //  G E T   M E D I C A L   R E C O R D   S U M M A R Y
  /**
   * <p>Accesses MedRec Web service retrieving all medical record summary.
   * This includes an array of abbreviated records
   * and an array of prescriptions.</p>
   *
   * @param pPatientId patient id.
   * @return RecordsSummary record summary
   * @throws PhysicianException
   */
  public RecordsSummary getRecordsSummary(Integer pPatientId)
      throws PhysicianException {
      
    logger.info("Getting records summary for patient id: "+
        pPatientId.toString());
    
    com.bea.medrec.domain.RecordsSummary recSummary = null;
    try {
      // Get record summary from MedRec.
      recSummary = syncMedRecWebServicesPortType.getRecordsSummary(
          pPatientId.intValue());
      logger.debug("Got record summary from MedRec.");
    } catch (Exception e) {
      logger.error(e);
      throw new PhysicianException("Unable to get record summary: "+
          e.getMessage(), e);
    }
    return toPhysicianRecordsSummary(recSummary);
  }

  //  S E A R C H   P A T I E N T
  /**
   * <p>Find patient by id.</p>
   *
   * @param pSearch Search value ojbect containing last name or ssn
   * @return List List of patients.
   * @throws PhysicianException
   */
  public List searchPatients(Search pSearch) throws PhysicianException {
    Patient patient = new Patient();
    if (isNotEmpty(pSearch.getLastName())) {
      patient.setLastName(pSearch.getLastName());
      return searchPatients(patient);
    } else if (isNotEmpty(pSearch.getSsn())) {
      patient.setSsn(pSearch.getSsn());
      return searchPatients(patient);
    } else {
      return Collections.EMPTY_LIST;
    }
  }

  //  S E A R C H   P A T I E N T
  /**
   * <p>Find patient by patient object.</p>
   *
   * @param pPatient Patient object containing last name
   * @return Patient  Patient object.
   * @throws PhysicianException
   */
  protected List searchPatients(Patient pPatient)
      throws PhysicianException {
    logger.info("Patient: "+pPatient.toString());

    if (isEmpty(pPatient.getLastName()) && isEmpty(pPatient.getSsn()))
      throw new PhysicianException("No search criteria specified.");

    // Declare local variables.
    com.bea.medrec.domain.Patient[] patients = null;
    try {
      // Get patients from MedRec.
     patients = syncMedRecWebServicesPortType.searchPatients(
          toMedRecPatient(pPatient));
      logger.debug("Got "+patients.length+" patients from MedRec.");
    } catch (Exception e) {
        e.printStackTrace();
      logger.error(e);
      throw new PhysicianException("Unable to search: "+
          e.getMessage(), e);
    }
    return Arrays.asList(toPhysicianPatients(patients));
  }
}